import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.core.client.ClientConsumer;
import org.hornetq.api.core.client.ClientMessage;
import org.hornetq.api.core.client.ClientProducer;
import org.hornetq.api.core.client.ClientSession;
import org.hornetq.api.core.client.ClientSessionFactory;
import org.hornetq.api.core.client.HornetQClient;
import org.hornetq.core.config.impl.FileConfiguration;
import org.hornetq.core.remoting.impl.invm.InVMConnectorFactory;
import org.hornetq.core.server.HornetQServer;
import org.hornetq.core.server.HornetQServers;
import org.junit.Test;

public class HornetQTest {
	private final int numMessages = 20;

	private final boolean autoCommitSends = true;

	// with autoCommitAcks=true the consumer threads gets stuck
	private final boolean autoCommitAcks = true;

	@Test
	public void testHornetQ() throws Exception {
		final FileConfiguration configuration = new FileConfiguration();

		configuration.start();

		final HornetQServer server = HornetQServers.newHornetQServer(configuration);

		server.start();
		try {
			TransportConfiguration transportConfiguration = new TransportConfiguration(
					InVMConnectorFactory.class.getName());

			final ClientSessionFactory sessionFactory = HornetQClient
					.createClientSessionFactory(transportConfiguration);
			try {
				ExecutorService executor = Executors.newCachedThreadPool();

				log("executing threads...");

				executor.submit(createProducer(sessionFactory));
				executor.submit(createConsumer(sessionFactory));

				executor.shutdown();

				log("awaiting threads...");

				executor.awaitTermination(Long.MAX_VALUE, TimeUnit.MINUTES);
			} finally {
				sessionFactory.close();
			}
		} finally {
			server.stop();
		}
	}

	private Runnable createConsumer(final ClientSessionFactory sessionFactory) {
		return new Runnable() {
			@Override
			public void run() {
				log("consumer started");

				try {
					ClientSession session = sessionFactory.createSession(autoCommitSends, autoCommitAcks);
					try {
						session.start();
						try {
							ClientConsumer consumer = session.createConsumer("jms.queue.TestQueue");
							try {
								for (int counter = 1; counter <= numMessages; counter++) {
									ClientMessage message = consumer.receive();

									log("consumer=" + counter);

									message.acknowledge();

									if (!autoCommitAcks)
										session.commit();
								}
							} finally {
								consumer.close();
							}
						} finally {
							session.stop();
						}
					} finally {
						session.close();
					}
				} catch (Exception exception) {
					log("consumer failed: " + exception);
				}

				log("consumer stopped");
			}
		};
	}

	private Runnable createProducer(final ClientSessionFactory sessionFactory) {
		return new Runnable() {
			@Override
			public void run() {
				log("producer started");

				try {
					ClientSession session = sessionFactory.createSession(autoCommitSends, autoCommitAcks);
					try {
						ClientProducer producer = session.createProducer("jms.queue.TestQueue");
						try {
							for (int counter = 1; counter <= numMessages; counter++) {
								ClientMessage message = session.createMessage(true);

								message.putIntProperty("counter", counter);

								producer.send(message);

								log("producer=" + counter);

								if (!autoCommitSends)
									session.commit();
							}
						} finally {
							producer.close();
						}
					} finally {
						session.close();
					}
				} catch (Exception exception) {
					log("producer failed: " + exception);
				}

				log("producer stopped");
			}
		};
	}

	private static void log(String message) {
		System.err.println(message);
	}
}
